import { AwsCustomResource } from "aws-cdk-lib/custom-resources";
import { Construct, IConstruct } from "constructs";
import { IChild, IParent } from "./parent";
import { IPolicyAttachmentTarget } from "./policy-attachment";
/**
 * @see https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/control-access-billing.html#ControllingAccessWebsite-Activate
 * @experimental
 */
export declare enum IamUserAccessToBilling {
    /**
     * (experimental) If set to ALLOW, the new account enables IAM users to access account billing information if they have the required permissions.
     *
     * @experimental
     */
    ALLOW = "ALLOW",
    /**
     * (experimental) If set to DENY, only the root user of the new account can access account billing information.
     *
     * @experimental
     */
    DENY = "DENY"
}
/**
 * @experimental
 */
export interface AccountBaseProps {
    /**
     * @experimental
     */
    readonly accountId?: string;
    /**
     * @experimental
     */
    readonly parent?: IParent;
    /**
     * @experimental
     */
    readonly email?: string;
    /**
     * @experimental
     */
    readonly accountName?: string;
    /**
     * @experimental
     */
    readonly roleName?: string;
    /**
     * @experimental
     */
    readonly iamUserAccessToBilling?: IamUserAccessToBilling;
}
/**
 * @experimental
 */
export interface AccountProps {
    /**
     * (experimental) The email address of the owner to assign to the new member account.
     *
     * This email address must not already be associated with another AWS account. You must use a valid email address to complete account creation. You can't access the root user of the account or remove an account that was created with an invalid email address.
     *
     * @experimental
     */
    readonly email: string;
    /**
     * (experimental) The friendly name of the member account.
     *
     * @experimental
     */
    readonly accountName: string;
    /**
     * (experimental) The name of an IAM role that AWS Organizations automatically preconfigures in the new member account.
     *
     * This role trusts the management account, allowing users in the management account to assume the role, as permitted by the management account administrator. The role has administrator permissions in the new member account.
     *
     * If you don't specify this parameter, the role name defaults to OrganizationAccountAccessRole.
     *
     * @experimental
     */
    readonly roleName?: string;
    /**
     * (experimental) If set to ALLOW , the new account enables IAM users to access account billing information if they have the required permissions.
     *
     * If set to DENY , only the root user of the new account can access account billing information.
     *
     * @default ALLOW
     * @experimental
     */
    readonly iamUserAccessToBilling?: IamUserAccessToBilling;
    /**
     * (experimental) The parent root or OU that you want to create the new Account in.
     *
     * @experimental
     */
    readonly parent?: IParent;
}
/**
 * @experimental
 */
export interface IAccount extends IChild, IConstruct {
    /**
     * (experimental) If the account was created successfully, the unique identifier (ID) of the new account.
     *
     * Exactly 12 digits.
     *
     * @experimental
     */
    readonly accountId: string;
    /**
     * (experimental) The Amazon Resource Name (ARN) of the account.
     *
     * @experimental
     */
    readonly accountArn: string;
    /**
     * (experimental) The friendly name of the account.
     *
     * @experimental
     */
    readonly accountName: string;
    /**
     * (experimental) The email address of the owner to assign to the new member account.
     *
     * This email address must not already be associated with another AWS account. You must use a valid email address to complete account creation. You can't access the root user of the account or remove an account that was created with an invalid email address.
     *
     * @experimental
     */
    readonly email: string;
}
/**
 * (experimental) Creates or imports an AWS account that is automatically a member of the organization whose credentials made the request.
 *
 * AWS Organizations automatically copies the information from the management account to the new member account
 *
 * @experimental
 */
export declare abstract class AccountBase extends Construct implements IAccount, IPolicyAttachmentTarget {
    /**
     * (experimental) If the account was created successfully, the unique identifier (ID) of the new account.
     *
     * Exactly 12 digits.
     *
     * @experimental
     */
    readonly accountId: string;
    /**
     * (experimental) The Amazon Resource Name (ARN) of the account.
     *
     * @experimental
     */
    readonly accountArn: string;
    /**
     * (experimental) The friendly name of the account.
     *
     * @experimental
     */
    readonly accountName: string;
    /**
     * (experimental) The email address of the owner to assign to the new member account.
     *
     * This email address must not already be associated with another AWS account. You must use a valid email address to complete account creation. You can't access the root user of the account or remove an account that was created with an invalid email address.
     *
     * @experimental
     */
    readonly email: string;
    /**
     * @experimental
     */
    protected readonly resource: AwsCustomResource;
    /**
     * @experimental
     */
    protected constructor(scope: Construct, id: string, props: AccountBaseProps);
    /**
     * (experimental) The unique identifier (ID) of the account or organizational unit (OU) that you want to retrieve the parent for.
     *
     * @experimental
     */
    identifier(): string;
    /**
     * @experimental
     */
    protected currentParent(): IParent;
    /**
     * @experimental
     */
    protected move(destinationParent: IParent): void;
    /**
     * @experimental
     */
    delegateAdministrator(servicePrincipal: string): void;
}
/**
 * @experimental
 */
export interface AccountAttributes {
    /**
     * @experimental
     */
    readonly accountId: string;
    /**
     * @experimental
     */
    readonly parent?: IParent;
}
/**
 * @experimental
 */
export declare class Account extends AccountBase {
    /**
     * (experimental) Import an existing account from account id.
     *
     * @experimental
     */
    static fromAccountId(scope: Construct, id: string, attrs: AccountAttributes): IAccount;
    /**
     * @experimental
     */
    constructor(scope: Construct, id: string, props: AccountProps);
}
