import { Construct, IConstruct } from "constructs";
import { IChild, IParent } from "./parent";
import { IPolicyAttachmentTarget } from "./policy-attachment";
/**
 * @experimental
 */
export interface OrganizationalUnitProps {
    /**
     * (experimental) The friendly name to assign to the new OU.
     *
     * @experimental
     */
    readonly organizationalUnitName: string;
    /**
     * (experimental) The parent root or OU that you want to create the new OrganizationalUnit in.
     *
     * @experimental
     */
    readonly parent: IParent;
}
/**
 * (experimental) A container for accounts within a root.
 *
 * An OU also can contain other OUs, enabling you to create a hierarchy that resembles an upside-down tree, with a root at the top and branches of OUs that reach down, ending in accounts that are the leaves of the tree. When you attach a policy to one of the nodes in the hierarchy, it flows down and affects all the branches (OUs) and leaves (accounts) beneath it. An OU can have exactly one parent, and currently each account can be a member of exactly one OU.
 *
 * <strong>You must first move all accounts out of the OU and any child OUs, and then you can delete the child OUs.</strong>
 *
 * @experimental
 */
export interface IOrganizationalUnit extends IPolicyAttachmentTarget, IParent, IChild, IConstruct {
    /**
     * (experimental) The unique identifier (ID) associated with this OU.
     *
     * The regex pattern for an organizational unit ID string requires "ou-" followed by from 4 to 32 lowercase letters or digits (the ID of the root that contains the OU). This string is followed by a second "-" dash and from 8 to 32 additional lowercase letters or digits.
     *
     * @experimental
     */
    readonly organizationalUnitId: string;
    /**
     * (experimental) The Amazon Resource Name (ARN) of this OU.
     *
     * For more information about ARNs in Organizations, see [ARN Formats Supported by Organizations](https://docs.aws.amazon.com/service-authorization/latest/reference/list_awsorganizations.html#awsorganizations-resources-for-iam-policies) in the AWS Service Authorization Reference.
     *
     * @experimental
     */
    readonly organizationalUnitArn: string;
    /**
     * (experimental) The friendly name of this OU.
     *
     * @experimental
     */
    readonly organizationalUnitName: string;
}
/**
 * @experimental
 */
export declare abstract class OrganizationalUnitBase extends Construct implements IOrganizationalUnit {
    /**
     * (experimental) The unique identifier (ID) associated with this OU.
     *
     * The regex pattern for an organizational unit ID string requires "ou-" followed by from 4 to 32 lowercase letters or digits (the ID of the root that contains the OU). This string is followed by a second "-" dash and from 8 to 32 additional lowercase letters or digits.
     *
     * @experimental
     */
    abstract readonly organizationalUnitId: string;
    /**
     * (experimental) The Amazon Resource Name (ARN) of this OU.
     *
     * For more information about ARNs in Organizations, see [ARN Formats Supported by Organizations](https://docs.aws.amazon.com/service-authorization/latest/reference/list_awsorganizations.html#awsorganizations-resources-for-iam-policies) in the AWS Service Authorization Reference.
     *
     * @experimental
     */
    abstract readonly organizationalUnitArn: string;
    /**
     * (experimental) The friendly name of this OU.
     *
     * @experimental
     */
    abstract readonly organizationalUnitName: string;
    /**
     * @experimental
     */
    identifier(): string;
}
/**
 * @experimental
 */
export interface OrganizationalUnitAttributes {
    /**
     * @experimental
     */
    readonly organizationalUnitId: string;
    /**
     * @experimental
     */
    readonly parent: IParent;
}
/**
 * @experimental
 */
export declare class OrganizationalUnit extends OrganizationalUnitBase {
    /**
     * @experimental
     */
    static fromOrganizationalUnitId(scope: Construct, id: string, attrs: OrganizationalUnitAttributes): IOrganizationalUnit;
    /**
     * (experimental) The unique identifier (ID) associated with this OU.
     *
     * The regex pattern for an organizational unit ID string requires "ou-" followed by from 4 to 32 lowercase letters or digits (the ID of the root that contains the OU). This string is followed by a second "-" dash and from 8 to 32 additional lowercase letters or digits.
     *
     * @experimental
     */
    readonly organizationalUnitId: string;
    /**
     * (experimental) The Amazon Resource Name (ARN) of this OU.
     *
     * For more information about ARNs in Organizations, see [ARN Formats Supported by Organizations](https://docs.aws.amazon.com/service-authorization/latest/reference/list_awsorganizations.html#awsorganizations-resources-for-iam-policies) in the AWS Service Authorization Reference.
     *
     * @experimental
     */
    readonly organizationalUnitArn: string;
    /**
     * (experimental) The friendly name of this OU.
     *
     * @experimental
     */
    readonly organizationalUnitName: string;
    /**
     * @experimental
     */
    constructor(scope: Construct, id: string, props: OrganizationalUnitProps);
    /**
     * @experimental
     */
    identifier(): string;
}
